/*
 * @file PropCS.c
 * This is the main PropCS program start point.
 *
*******************************************************************************
 */

#include <stdio.h>
#include <propeller.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/rtc.h>
#include <cog.h>
#include <ctype.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/sd.h>

extern _Driver _SimpleSerialDriver;
extern _Driver _FileDriver;

/* This is a list of all drivers we can use in the
 * program. The default _InitIO function opens stdin,
 * stdout, and stderr based on the first driver in
 * the list (the serial driver, for us)
 */
_Driver *_driverlist[] = {
  &_SimpleSerialDriver,
  &_FileDriver,
  NULL
};

FILE *stdinfile;
FILE *stdoutfile;

void Help()
{
    printf("Commands are 'help, time, settime, ls, cat, rm, ll, echo, cd, pwd, mkdir, rmdir  '\n");
    printf("Manual pages 'mantime, mansettime, manls, mancat, manrm, manll, manecho,\n");
    printf("mancd, manpwd, manmkdir, manrmdir, manmisc\n");
}

/*
 * Main program function.
 */
int main(void)
{
    int num;
    char *tokens[20];
    uint8_t buffer[80];

    stdinfile = stdin;
    stdoutfile = stdout;

    _rtc_start_timekeeping_cog();

    waitcnt(CNT + CLKFREQ);
    mount();
    printf("This is the PropCS program!\n");
    printf("Type help for 'Commands' list.\n");

    while(1)
    {
        printf(">");
        fflush(stdout);
        gets(buffer);
        num = tokenize(buffer, tokens);
        num = CheckRedirection(tokens, num);
        if (num == 0) continue;
        if (!strcmp(tokens[0], "help"))
            Help();
        else if (!strcmp(tokens[0], "time"))
            PrintTime();
        else if (!strcmp(tokens[0], "settime"))
            SetTime();
        else if (!strcmp(tokens[0], "ls"))
            List(num, tokens);
        else if (!strcmp(tokens[0], "cat"))
            Cat(num,tokens);
        else if (!strcmp(tokens[0], "ll"))
        {
            tokens[num++] = "-l";
            List(num, tokens);
        }
        else if (!strcmp(tokens[0], "rm"))
            Remove(num, tokens);
        else if (!strcmp(tokens[0], "echo"))
            Echo(num, tokens);
        else if (!strcmp(tokens[0], "cd"))
            Cd(num, tokens);
        else if (!strcmp(tokens[0], "pwd"))
            Pwd(num, tokens);
        else if (!strcmp(tokens[0], "mkdir"))
            Mkdir(num, tokens);
        else if (!strcmp(tokens[0], "rmdir"))
            Rmdir(num, tokens);
        else if (!strcmp(tokens[0], "mantime"))
            ManTime();
        else if (!strcmp(tokens[0], "mansettime"))
            ManSetTime();
        else if (!strcmp(tokens[0], "mancat"))
            ManCat();
        else if (!strcmp(tokens[0], "manrm"))
            ManRm();
        else if (!strcmp(tokens[0], "manls"))
            ManLs();
        else if (!strcmp(tokens[0], "manll"))
            ManLL();
        else if (!strcmp(tokens[0], "manecho"))
            ManEcho();
        else if (!strcmp(tokens[0], "mancd"))
            ManCD();
        else if (!strcmp(tokens[0], "manmkdir"))
            ManMkdir();
        else if (!strcmp(tokens[0], "manrmdir"))
            ManRmdir();
        else if (!strcmp(tokens[0], "manmisc"))
            ManMisc();
        else
        {
            printf("Invalid command\n");
        }
        CloseRedirection();
    }
    return 0;
}

