/* PropCS_SD.c */

#include <stdio.h>
#include <propeller.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/rtc.h>
#include <cog.h>
#include <ctype.h>
#include <sys/stat.h>
#include <dirent.h>
#include <sys/sd.h>

extern _Driver _SimpleSerialDriver;
extern _Driver _FileDriver;

_Driver *driverlist[] = {
    &_SimpleSerialDriver,
    &_FileDriver,
    NULL
};

FILE *stdinfile;
FILE *stdoutfile;

char *FindChar(char *ptr, int val);

void Cd(int argc, char **argv)
{
    if (argc < 2) return;

    if (chdir(argv[1]))
        perror(argv[1]);
}

void Pwd(int argc, char **argv)
{
    uint8_t buffer[64];
    char *ptr = getcwd(buffer, 64);
    if (!ptr)
        perror(0);
    else
        fprintf(stdoutfile, "%s\n", ptr);
}

void Mkdir(int argc, char **argv)
{
    int i;

    for (i = 1; i < argc; i++)
    {
        if (mkdir(argv[i], 0))
            perror(argv[i]);
    }
}

void Rmdir(int argc, char **argv)
{
    int i;

    for (i = 1; i < argc; i++)
    {
        if (rmdir(argv[i]))
            perror(argv[i]);
    }
}

/* This routine implements the file cat function */
void Cat(int argc, char **argv)
{
    int i;
    int num;
    void *infile;
    uint8_t buffer[40];

//    for (i = 0; i < argc; i++)
    for (i=1; i < argc; i++)
    {
        if (i == 0)
        {
            if (argc == 1 || stdinfile != stdin)
                infile = stdinfile;
            else
                continue;
        }
        else
        {
            infile = fopen(argv[i], "r");
            if (infile == 0)
            {
                perror(argv[i]);
                continue;
            }
        }
        if (infile == stdin)
        {
            while (gets(buffer))
            {
                if (buffer[0] == 4) break;
                fprintf(stdoutfile, "%s\n", buffer);
            }
        }
        else
        {
            while ((num = fread(buffer, 1, 40, infile)))
                fwrite(buffer, 1, num, stdoutfile);
        }
        if (i)
            fclose(infile);
    }
    fflush(stdout);
}

/* This routine deletes the files specified by the command line arguments */
void Remove(int argc, char **argv)
{
    int i;

    for (i = 1; i < argc; i++)
    {
        if (remove(argv[i]))
            perror(argv[i]);
    }
}

/* This routine echos the command line arguments */
void Echo(int argc, char **argv)
{
    int i;
    for (i = 1; i < argc; i++)
    {
        if (i != argc - 1)
            fprintf(stdoutfile, "%s ", argv[i]);
        else
            fprintf(stdoutfile, "%s\n", argv[i]);
    }
}

/* This routine lists the root directory or any subdirectories specified
   in the command line arguments.  If the "-l" option is specified, it
   will print the file attributes and size.  Otherwise, it will just
   print the file names.  */
void List(int argc, char **argv)
{
    int i, j;
    char *ptr;
    char fname[13];
    int32_t count = 0;
    uint32_t filesize;
    uint32_t longflag = 0;
    char *path;
    char drwx[5];
    int column;
    int prevlen;
    DIR *dirp;
    struct dirent *entry;

    // Check flags
    for (j = 1; j < argc; j++)
    {
        if (argv[j][0] == '-')
        {
            if (!strcmp(argv[j], "-l"))
                longflag = 1;
            else
                printf("Unknown option \"%s\"\n", argv[j]);
        }
        else
            count++;
    }

    // List directories
    for (j = 1; j < argc || count == 0; j++)
    {
        if (count == 0)
        {
            count--;
            path = "./";
        }
        else if (argv[j][0] == '-')
            continue;
        else
            path = argv[j];

        if (count >= 2)
            fprintf(stdoutfile, "\n%s:\n", path);

        dirp = opendir(path);

        if (!dirp)
        {
            perror(path);
            continue;
        }

        column = 0;
        prevlen = 14;
        while (entry = readdir(dirp))
        {
            if (entry->name[0] == '.') continue;
            ptr = fname;
            for (i = 0; i < 8; i++)
            {
                if (entry->name[i] == ' ') break;
                *ptr++ = tolower(entry->name[i]);
            }
            if (entry->name[8] != ' ')
            {
                *ptr++ = '.';
                for (i = 8; i < 11; i++)
                {
                    if (entry->name[i] == ' ') break;
                    *ptr++ = tolower(entry->name[i]);
                }
            }
            *ptr = 0;
            filesize = entry->filesize_3;
            filesize = (filesize << 8) | entry->filesize_2;
            filesize = (filesize << 8) | entry->filesize_1;
            filesize = (filesize << 8) | entry->filesize_0;
            strcpy(drwx, "-rw-");
            if (entry->attr & ATTR_READ_ONLY)
                drwx[2] = '-';
            if (entry->attr & ATTR_ARCHIVE)
                drwx[3] = 'x';
            if (entry->attr & ATTR_DIRECTORY)
            {
                drwx[0] = 'd';
                drwx[3] = 'x';
            }
            if (longflag)
                fprintf(stdoutfile, "%s %8d %s\n", drwx, filesize, fname);
            else if (++column == 5)
            {
                for (i = prevlen; i < 14; i++) fprintf(stdoutfile, " ");
                fprintf(stdoutfile, "%s\n", fname);
                column = 0;
                prevlen = 14;
            }
            else
            {
                for (i = prevlen; i < 14; i++) fprintf(stdoutfile, " ");
                prevlen = strlen(fname);
                fprintf(stdoutfile, "%s", fname);
            }
        }
        closedir(dirp);
        if (!longflag && column)
            fprintf(stdoutfile, "\n");
    }
}



/* This routine returns a pointer to the first character that doesn't
   match val. */
char *SkipChar(char *ptr, int val)
{
    while (*ptr)
    {
        if (*ptr != val) break;
        ptr++;
    }
    return ptr;
}

/* This routine returns a pointer to the first character that matches val. */
char *FindChar(char *ptr, int val)
{
    while (*ptr)
    {
        if (*ptr == val) break;
        ptr++;
    }
    return ptr;
}


/* This routine extracts tokens from a string that are separated by one or
   more spaces.  It returns the number of tokens found. */
int tokenize(char *ptr, char *tokens[])
{
    int num = 0;

    while (*ptr)
    {
        ptr = SkipChar(ptr, ' ');
        if (*ptr == 0) break;
        if (ptr[0] == '>')
        {
            ptr++;
            if (ptr[0] == '>')
            {
                tokens[num++] = ">>";
                ptr++;
            }
            else
                tokens[num++] = ">";
            continue;
        }
        if (ptr[0] == '<')
        {
            ptr++;
            tokens[num++] = "<";
            continue;
        }
        tokens[num++] = ptr;
        ptr = FindChar(ptr, ' ');
        if (*ptr) *ptr++ = 0;
    }
    return num;
}


/* This routine searches the list of tokens for the redirection operators
   and opens the files for input, output or append depending on the 
   operator. */
int CheckRedirection(char **tokens, int num)
{
    int i, j;

    for (i = 0; i < num-1; i++)
    {
        if (!strcmp(tokens[i], ">"))
        {
            stdoutfile = fopen(tokens[i+1], "w");
            if (!stdoutfile)
            {
                perror(tokens[i+1]);
                stdoutfile = stdout;
                return 0;
            }
        }
        else if (!strcmp(tokens[i], ">>"))
        {
            stdoutfile = fopen(tokens[i+1], "a");
            if (!stdoutfile)
            {
                perror(tokens[i+1]);
                stdoutfile = stdout;
                return 0;
            }
        }
        else if (!strcmp(tokens[i], "<"))
        {
            stdinfile = fopen(tokens[i+1], "r");
            if (!stdinfile)
            {
                perror(tokens[i+1]);
                stdinfile = stdin;
                return 0;
            }
        }
        else
            continue;
        for (j = i + 2; j < num; j++) tokens[j-2] = tokens[j];
        i--;
        num -= 2;
    }
    return num;
}


/* This routine closes files that were open for redirection */
void CloseRedirection()
{
    if (stdinfile != stdin)
    {
        fclose(stdinfile);
        stdinfile = stdin;
    }
    if (stdoutfile != stdout)
    {
        fclose(stdoutfile);
        stdoutfile = stdout;
    }
}

void mount()
{
    printf("Load and mount SD: ");
    _SD_Params* mountParams = (_SD_Params*)-1;

// Important: This code assumes you're using a C3 card.
// If you're using different hardware, make sure you
// change the following initialization to match your card!

#ifdef SPINNERET_CARD
    static _SD_Params params =
    {
        AttachmentType: _SDA_SingleSPI,
        pins:
        {
            SingleSPI:
            {
                MISO: 16,   // The pin attached to the SD card's MISO or DO output
                CLK:  21,   // The pin attached to the SD card's CLK or SCLK input
                MOSI: 20,   // The pin attached to the SD card's MOSI or DI input
                CS:   19    // The pin attached to the SD card's CS input
            }
        }
    };
    mountParams = &params;
#endif
#ifdef PROP_BOE /* Board of Education */
    static _SD_Params params =
    {
        AttachmentType: _SDA_SingleSPI,
        pins:
        {
            SingleSPI:
            {
                MISO: 22,   // The pin attached to the SD card's MISO or DO output
                CLK:  23,   // The pin attached to the SD card's CLK or SCLK input
                MOSI: 24,   // The pin attached to the SD card's MOSI or DI input
                CS:   25    // The pin attached to the SD card's CS input
            }
        }
    };
    mountParams = &params;
#endif

#ifdef PPUSB
    static _SD_Params params =
    {
        AttachmentType: _SDA_SingleSPI,
        pins:
        {
            SingleSPI:
            {
                MISO: 0,   // The pin attached to the SD card's MISO or DO output
                CLK:  1,   // The pin attached to the SD card's CLK or SCLK input
                MOSI: 2,   // The pin attached to the SD card's MOSI or DI input
                CS:   3    // The pin attached to the SD card's CS input
            }
        }
    };
    mountParams = &params;
#endif


#ifdef C3_CARD

    static _SD_Params params =
    {
        AttachmentType: _SDA_SerialDeMUX,
        pins:
        {
            SerialDeMUX:
            {
                MISO: 10,    // The pin attached to the SD card's MISO or DO output
                CLK:  11,    // The pin attached to the SD card's CLK or SCLK input
                MOSI: 9,     // The pin attached to the SD card's MOSI or DI input
                CLR:  25,    // The pin attached to the counter's reset/clear pin
                INC:  8,     // The pin attached to the counter's clock/count pin
                ADDR: 5,     // The SD card's demux address (the counter's count)
            }
        }
    };
    mountParams = &params;
#endif
//#define SD_IS_USING_SD_CACHE_DIVER

#ifdef PARALLEL_SPI /* This is a hypothetical example - modify to suit your needs */
    static _SD_Params params =
    {
        AttachmentType: _SDA_ParallelDeMUX,
        pins:
        {
            ParallelDeMUX:
            {
                MISO: 4,    // The pin attached to the SD card's MISO or DO output
                CLK:  5,    // The pin attached to the SD card's CLK or SCLK input
                MOSI: 6,    // The pin attached to the SD card's MOSI or DI input
                CS:   0,    // The pin attached to the counter's reset/clear pin
                SEL:  0x04, // The pin mask to set when selecting the SD card's deMUX address
                MSK:  0x0E  // The pin mask for all pins attached to the deMUX address
            }
        }
    };
    mountParams = &params;
#endif

#if defined(__PROPELLER_XMMC__) && defined(SD_IS_USING_SD_CACHE_DRIVER)
    // Pass NULL as the params. In this case, we'll use the SD Cache driver.
    // Beware: This only works if you're running your program
    // cached off of the SD card (i.e. propeller-load -z).
    mountParams = 0;
#endif

    if (mountParams == (_SD_Params*)-1)
    {
        printf("You must specify the SD paramters in the filetest.c\n");
        exit(1);
    }

    uint32_t mountErr = dfs_mount(mountParams);
    if (mountErr)
    {
        printf("Mount error: %d\n", mountErr);
        exit(1);
    }

    printf("done.\n\n");
}

void ManCat()
{
    printf("cat - opens and displays contents of a file.\n");
    printf(" cat test1.txt\n"); 
}

void ManRm()
{
    printf("rm - removes (erases) a file.\n");
    printf(" rm test1.txt\n");
}

void ManLs()
{
    printf("ls - list files and directories.\n");
    printf("ls -l - list files with size and attributes.\n");
}

void ManLL()
{
    printf("ll - list files with size and attributes.\n");
}

void ManEcho()
{
    printf("echo - echos the command line arguments.\n");
    printf(" echo This is a line of text. Will display 'This is a line of text.'\n");
    printf(" echo This is a line of text. >> test1.txt\n");
    printf(" Will create the test1.txt file with 'This is a line of text' in the file.\n");
}

void ManCD()
{
    printf("cd - change directory.\n");
    printf(" cd .. - change to root directory.\n");
}

void ManMkdir()
{
    printf("mkdir - create a directory.\n");
    printf(" mkdir test - creates directory 'test'.\n");
}

void ManRmdir()
{
    printf("rmdir - remove a directory.\n");
    printf(" rmdir test - removes the directory 'test'.\n");
}

void ManMisc()
{
    printf("File redirection - '<, >, <<, >>'\n");
}